#ifndef __CINT__
#define ISDCMAIN ghost_busters
#endif
#include "scriptstuff.h"
#include "assert.h"
#include "TMinuit.h"

const double cm2perpixel=0.4*0.4;
const double saclaynorm=cm2perpixel*128.0*128.0/2.0;

int numm=0;


const double Zdiff=0;
const double Ydiff=0;

const int IBIS_SCRIPT_ERROR=-150000;
const int ERROR_CODE_START=IBIS_SCRIPT_ERROR;

const int MASKINFITS=3*159;
const int NPIXY=130;
const int NPIXZ=134;
const int NMODY=3510;
const int NMODZ=3618;
const int divpix=9;

const int PIXSIZE=4;

double OFFY=0.241129;
double OFFZ=-0.114642; 

double alpha=0.0;
//double MDD=3208;/* mm center of mask to center of detector */

double MDDY=3202.967;
double MDDZ=MDDY;


double DTH=0.0;/*thickness of pixel */
double SPIDER=25; /* Spider */ 

const double DPS=4.6;/* mm pixel pitch */
const double MPS=11.2/3.0;/* mm mask pitch, scaled by 3 for expanded mask */
double MTH=16;/* mm mask thickness */

const double BIG=1E30;

const double rad2deg=3.14159265358979323846/180.0;

const double norm=0.930649;
//const double norm=1.0;

int calculatelocal(double ra,double dec,double rax,double decx,
                   double raz,double decz, double *yloc,double *zloc, int status)
		   {
  double s[3];
  double x[3];
  double z[3];
  double y[3];
  double perp;
  double tol=2e-6; /* changed after seen real example of 1.000001E-6 */ 
  do{
    slaDcs2c(rad2deg*ra,rad2deg*dec,s);
    slaDcs2c(rad2deg*rax,rad2deg*decx,x);
    slaDcs2c(rad2deg*raz,rad2deg*decz,z);

    double cosangle=s[0]*x[0]+s[1]*x[1]+s[2]*x[2];
	 double alpha=acos(cosangle);
    if (cosangle<0.9){
	    *yloc=10000;
	    *zloc=10000;
    }
    else {
    y[0]=z[1]*x[2]-z[2]*x[1];
    y[1]=z[2]*x[0]-z[0]*x[2];
    y[2]=z[0]*x[1]-z[1]*x[0];
    
    *yloc=s[0]*y[0]+s[1]*y[1]+s[2]*y[2];
//    *yloc=asin(*yloc);
//    *yloc=-MDDY/DPS*tan(*yloc);
	 *yloc=-MDDY/DPS*(*yloc)/cosangle;

    *zloc=s[0]*z[0]+s[1]*z[1]+s[2]*z[2];
//    *zloc=asin(*zloc);
//    *zloc=MDDZ/DPS*tan(*zloc);
	 *zloc=MDDZ/DPS*(*zloc)/cosangle;
    }
    /*printf("YFIN,ZFIN: %g %g\n",*yloc,*zloc);
    printf("RASCX,DECSCX: %g %g\n",rad2deg*rax,rad2deg*decx);*/
	 
    }
    while (0);
    return status;
}

  
int ghost_busters(int argc=0,char **argv=0){
  using namespace ISDC;
  int status=ISDC_OK;
  do{
    ISDCTask ghost_busters("ghost_busters","1.0", argc, argv, ISDCTask::kCLI);
    status=ghost_busters.GetStatus();
    if (status != ISDC_OK) break;
    dal_element *og=NULL;
    int clobber;
    status=CommonPreparePARsStrings("inDOL","","ogDOL","",0,&og,&clobber,status);
    if (status!=ISDC_OK) break;
    bool align=(bool)ghost_busters["align"];
//    int nsource=ghost_busters["nsource"];
    	    
    double rax,decx,raz,decz;
    {
      double nrax,ndecx,nraz,ndecz;
      double time,time1;
      status=DALattributeGetReal(og,"TSTART",&time,NULL,NULL,status);
      if (status!=ISDC_OK)break;
      status=DALattributeGetReal(og,"TSTOP",&time1,NULL,NULL,status);
      time+=time1;
      time/=2.0;
      if (status!=ISDC_OK)break;
      status=DAL3AUXgetAttitude(og,DAL3AUX_ANY,1,&time,&nrax,&ndecx,&nraz,&ndecz,status);
      if (status!=ISDC_OK)break;
      rax=nrax;/* if we want to bypass alignement*/
      decx=ndecx;
      raz=nraz;
      decz=ndecz;
		if (align){
      	status=DAL3AUXcalcInstrumentAttitude(nrax,ndecx,nraz,ndecz,time,ISGRI,&rax,&decx,&raz,&decz,status);
      	if (status!=ISDC_OK)break;
		}
     
    }
	 
	 dal_element* x=NULL;
	 status=DALobjectOpen((const char *)ghost_busters["maskmod"],&x,status);
    if (status!=ISDC_OK)break;
	 dal_dataType t1; 
	 int numaxmask;
	 long axmask[2];
	 status=DALarrayGetStruct(x,&t1,&numaxmask,axmask,status);
	 if (status!=ISDC_OK)break;
	 long stmask[2]={1,1};
	 long nm;
	 int* mask=new int[axmask[0]*axmask[1]];
	 t1=DAL_INT;
	 status=DALarrayGetSection(x,2,stmask,axmask,&t1,&nm,mask,status);
	 if (status!=ISDC_OK)break;
	 status=DALobjectClose(x,DAL_SAVE,status);
	 if (status!=ISDC_OK)break;
	 
    dal_element *zz=NULL;
    status=DALobjectOpen((const char *)ghost_busters["sourcecat"],&zz,status);
    if (status!=ISDC_OK)break;
    double *ra=NULL;
    double *dec=NULL;
    double *yloc=NULL;
    double *zloc=NULL;
    char **name=NULL;
    long nsource;
    
    dal_dataType tt=DAL_DOUBLE;
    long bla;
    status=DALtableGetNumRows(zz,&(nsource),status);
    if (status!=ISDC_OK) break;
    if (nsource==0){
        RILlogMessage(NULL,Log_0,"No sources in input file, we will not kill screws");
	 }
    else RILlogMessage(NULL,Log_0,"Found %i sources with screws to kill",nsource);
    
    if (nsource>0){
	   ra=new double[nsource];
	 	dec=new double[nsource];
	 	yloc=new double[nsource];
	 	zloc=new double[nsource];
	 	name=new char*[nsource];
      for (int i=0;i<nsource;i++){
      	name[i]=new char[21];
      	name[i][0]=0;
      }
	 	status=DALtableGetCol(zz,"RA_OBJ",1,&tt,&(bla),(void *)ra,status);
	 	if (status!=ISDC_OK) break;
	 	status=DALtableGetCol(zz,"DEC_OBJ",1,&tt,&(bla),(void *)dec,status);
	 	if (status!=ISDC_OK) break;
	 	status=DALtableGetColStrings(zz,"NAME",1,1,(long)nsource,&(bla),name,status);
	 	if (status!=ISDC_OK) break;
	}
   status=DALobjectClose(zz,DAL_SAVE,status);
   if (status!=ISDC_OK) break;
   int nband;
   dal_element *kk=NULL;
   status=DAL3GENindexFindMember(og,"ISGR-CEXP-SHD",NULL,&nband,&kk,status);
   nband/=3;
   if (status==-25103){
     RILlogMessage(NULL,Warning_0,"No structures ISGR-CEXP-SHD found");
   }
   if (status!=ISDC_OK) break;
   RILlogMessage(NULL,Log_0,"Found %d ISGR-CEXP-SHD",nband);
   for (int ishad=0;ishad<nband;ishad++){
   	long start[2];
		long end[2];
		double *var=new double[NPIXZ*NPIXY];
		double *eff=new double[NPIXZ*NPIXY];
		dal_element *v=NULL;
		dal_element *e=NULL;
      status=DAL3GENindexGetMember(kk,"ISGR-CEXP-SHD",ishad*3+2,&v,status);
      if (status!=ISDC_OK) break;
      status=DAL3GENindexGetMember(kk,"ISGR-CEXP-SHD",ishad*3+3,&e,status);
      if (status!=ISDC_OK) break;
      int numaxes;
      status=DALarrayGetStruct(v,&tt,&numaxes,end,status);
      assert(numaxes==2);
      assert(end[0]==NPIXY);
      assert(end[1]==NPIXZ);
      status=DALarrayGetStruct(e,&tt,&numaxes,end,status);
      if (status!=ISDC_OK) break;
      assert(numaxes==2);
      assert(end[0]==NPIXY);
      assert(end[1]==NPIXZ);
      tt=DAL_DOUBLE;
      start[0]=1;
      start[1]=1;
      status=DALarrayGetSection(v,2,start,end,&tt,&bla,(void *)var,status);
      if (status!=ISDC_OK) break;
      status=DALarrayGetSection(e,2,start,end,&tt,&bla,(void *)eff,status);
      if (status!=ISDC_OK) break;
   
	/* kill screws */
		for (int ns=0;ns<nsource;ns++){
			status=calculatelocal(ra[ns],dec[ns],rax,decx,raz,decz,&(yloc[ns]),&(zloc[ns]),ISDC_OK);
			if ((fabs(yloc[ns]>160.))||(fabs(zloc[ns]>160.))){
				RILlogMessage(NULL,Log_0,"Source no %i: %s outside the FOV",ns,name[ns]);
			}
			else {
			RILlogMessage(NULL,Log_0,"Source no %i: %s",ns,name[ns]);
			RILlogMessage(NULL,Log_0,"RA, Dec of source %i : %f, %f",ns,ra[ns],dec[ns]);
			RILlogMessage(NULL,Log_0,"Detector position of source %d : %f, %f",ns,yloc[ns],zloc[ns]);
      	for (int z=0; z<NPIXZ; z++) {
	   		for (int y=0; y<NPIXY; y++) {
         		int my1=(int)floor((y-NPIXY/2.0-yloc[ns])*DPS/MPS+MASKINFITS/2.0); //coin en bas  gauche
         		int mz1=(int)floor((z-NPIXZ/2.0+zloc[ns])*DPS/MPS+MASKINFITS/2.0);
					int my2=(int)floor((y-NPIXY/2.0-yloc[ns])*DPS/MPS+MASKINFITS/2.0+1); //coin en haut  droite
					int mz2=(int)floor((z-NPIXZ/2.0+zloc[ns])*DPS/MPS+MASKINFITS/2.0+1);
					
					// pixels with a value of 2 are ignored
					if ((mz1*axmask[0]+my1>=0)&&(mz2*axmask[0]+my2<axmask[0]*axmask[1])){
						if ((mask[mz1*axmask[0]+my1]==2)||(mask[mz1*axmask[0]+my2]==2)||(mask[mz2*axmask[0]+my1]==2)||(mask[mz2*axmask[0]+my2]==2)){
                  	eff[z*NPIXY+y]=0.0;
							var[z*NPIXY+y]=0.0;
						}
					}
						
         	}
      	}
			}
		}
 	   status=DALarrayPutSection(v,2,start,end,DAL_DOUBLE,(void *)var,status);
   	if (status!=ISDC_OK) break;
 	   status=DALarrayPutSection(e,2,start,end,DAL_DOUBLE,(void *)eff,status);
   	if (status!=ISDC_OK) break;
		delete [] var;
		delete [] eff;
	}
	status=DALobjectClose(og,DAL_SAVE,ISDC_OK);
	if (status!=ISDC_OK) break;
	for (int i=0;i<nsource;i++){
		delete [] name[i];
	}
	delete [] name;
	delete [] ra;
	delete [] dec;
	delete [] yloc;
	delete [] zloc;
  }while (0);
  return status;
}
